import pathlib
import os
def __reset_files():
    """
     file rester
    """
    _current_abso_path = pathlib.Path(__file__).parent.parent.absolute()
    _inner_lcode_src = open(_current_abso_path.joinpath('_src/__code_sr_list__.py'), 'r')
    _raw = _inner_lcode_src.read()
    _inner_lcode_src.close()
    os.remove(_current_abso_path.joinpath('_gen/__code_gen_list__.py'))
    _inner_lcode_gen = open(_current_abso_path.joinpath('_gen/__code_gen_list__.py'), 'w')
    _inner_lcode_gen.write(_raw)
    _inner_lcode_gen.close()


import sys
import traceback
from importlib import reload
import pathlib
from tocode.__inner_files__._built_in._built_in_file_manager import __reset_files
from tocode._literal_eval_par._str_parser import __Str

_current_abso_path = pathlib.Path(__file__).parent.parent.absolute()

def literal_eval(str_list: str, no_string_quotation: bool = False):
    """
    the list parser will convert list in string to the list and there is
    an option available `no_string_quotation` if you string contains the
    needed quotations like bellow:
        Example = "['hello', 5, 7, ('damage', 5)]"
    make it 'False' or leave it because default is False.

    and when ever your strings don't contain any quotation make this 'True'
    it will add quotation for you like bellow:
        Example = [This is example, 9, [[[hello, 4]]]]
    :param str_list: the list in string which must convert to actual python list
    :param no_string_quotation: does the strings contain quotation?
    :return: actual python list converted from str_list
    """
    __reset_files()
    _lcd = ''
    if no_string_quotation:
        str_list = __Str.space_encoder(str_list)
        str_list_final = str_list
        _addi_index = 0
        while True:
            try:
                exec(str_list)
            except SyntaxError as err:
                error_class = err.__class__.__name__
                detail = err.args[0]
                line_number = err.lineno
            except Exception as err:
                error_class = err.__class__.__name__
                detail = err.args[0]
                cl, exc, tb = sys.exc_info()
                line_number = traceback.extract_tb(tb)[-1][1]
            else:
                break

            try:
                if detail.split('\'')[0] == 'name ' and detail.split('\'')[2] == ' is not defined':
                    _f_index = str_list.find(detail.split('\'')[1])
                    str_list = str_list.replace(detail.split('\'')[1], "'" + 'त'*(len(detail.split('\'')[1])) + "'", 1)
                    str_list_final = str_list_final[:_f_index] + "'" + str_list_final[_f_index: (_f_index + len(detail.split('\'')[1]))] + "'" + str_list_final[(_f_index + len(detail.split('\'')[1])):]
                else:
                    return detail + ' at line ' + str(line_number)
            except Exception as ignored:
                return detail + ' at line ' + str(line_number)
        str_list = __Str.space_decoder(str_list_final)

    with open(_current_abso_path.joinpath('__inner_files__/_gen/__code_gen_list__.py'), 'r') as _inner_lcode_gen:
        _inner_lcode_gen = open(_current_abso_path.joinpath('__inner_files__/_gen/__code_gen_list__.py'), 'r')
        _lcd = _inner_lcode_gen.read()

    with open(_current_abso_path.joinpath('__inner_files__/_gen/__code_gen_list__.py'), 'w') as _inner_lcode_gen:
        _lcd = _lcd.replace('return', 'return ' + str_list)
        _inner_lcode_gen.write(_lcd)

    from tocode.__inner_files__._gen import __code_gen_list__
    __code_gen_list__ = reload(__code_gen_list__)

    _generated_arr = __code_gen_list__._list_code__gen()
    return _generated_arr



from setuptools import find_packages, setup

with open('README.md') as f:
    long_description = f.read()

with open('doc/Version') as ver:
    version_ = ver.read()

setup(
    name='tocode',
    packages=find_packages(include=['tocode.*']),

    version=version_,
    description='a python library that convert string to code.',
    author='Amir Shamsi',
    url='https://github.com/Amir-Shamsi/tocode',

    license='MIT',
    author_email='amirshamsi.github@gmail.com',

    github='https://github.com/Amir-Shamsi',
    linkedin='https://linkedin.com/in/amir-shamsi',

    install_requires=[],
    download_url='https://github.com/Amir-Shamsi/tocode/archive/refs/tags/'+ version_ +'.tar.gz',

    keywords=['tocode', 'convertor', 'string-to-code', 'code', 'string', 'code-generator', 'generator'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests',
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown'
)




__inner_signs__ = ['{', '}', '[', ']', '(', ')', ',', '', ':']
class __Str:
    """
    the class contain string methods
    """
    @staticmethod
    def space_encoder(_str: str):
        """
        will encode spaces inside list in string.
        :param _str: list in string
        :return: return encoded string
        """
        _numeric_flag = False
        _str = list(_str)
        _space_index_st = 0
        for _index in range(len(_str)):
            if _str[_index] == ' ':
                if _str[_index - 1] not in __inner_signs__ and _str[_index + 1] not in __inner_signs__ and not _numeric_flag:
                    _str[_index] = 'र'
                    if _space_index_st == 0:
                        _space_index_st = _index
                else:
                    _str[_index] = ''
            else:
                if _str[_index] in __inner_signs__:
                    _numeric_flag = True
                if (not _str[_index].isnumeric()) and _str[_index] != ' ' and _str[_index] != '' and (_str[_index] not in __inner_signs__):
                    _numeric_flag = False

                if _space_index_st > 0 and _str[_index] == ',':
                    for _ind in range(_space_index_st, _index):
                        _str[_ind] = ''

                _space_index_st = 0
        return ''.join(_str)

    @staticmethod
    def space_decoder(_str: str):
        """
        will decode spaces inside list in string.
        :param _str: list in string
        :return: return encoded string
        """
        _str = list(_str)
        for _index in range(len(_str)):
            if _str[_index] == 'र':
                _str[_index] = ' '
        return ''.join(_str)
